#include <qwt_math.h>
#include <qdrawutl.h>
#include <qwt_wheel.h>

//------------------------------------------------------------
//.H QwtWheel | 3 | 27/04/96 | Qwt Widget Library | Qwt Programmer's Manual
//.I wheel.gif
//.S NAME
//	QwtWheel - The Wheel Widget
//
//.S SYNOPSIS
//	#include <qwt_wheel.h>
//
//.S INHERITED CLASSES
//     \l{qwr_sldbase}{QwtSliderBase}
//
//.S PUBLIC MEMBERS
//.t
//	void \R{QwtWheel::setTotalAngle}(double angle) --
//		Set the maximum angle by which the wheel can be turned
//	void \R{QwtWheel::setTickCnt}(int cnt} --
//		set the number of grooves in the wheel's surface 
//	void \R{QwtWheel::setOrientation}(Orientation o) -- Change the orientation
//	void \R{QwtWheel::setViewAngle}(double angle) -- Change the visible angle
//	void \R{QwtWheel::setInternalBorder}(int width) -- Change the internal border
//	void \R{QwtWheel::setMass}(double val) -- Set the mass for a flywheel effect
//
//.S DESCRIPTION
//	The wheel widget can be used to change values over a very large range
//	in very small steps. Using the setMass member, it can be configured
//	as a flywheel.
//.S EXAMPLE
//	See the radio example.
//------------------------------------------------------------


//------------------------------------------------------------
//
//.S	QwtWheel::QwtWheel
//	Constructor
//
//.s	Parameters
//.p	QWidget *parent, const char *name
//
//.s	Syntax
//.f	 QwtWheel::QwtWheel(QWidget *parent, const char *name)
//
//------------------------------------------------------------
QwtWheel::QwtWheel(QWidget *parent, const char *name)
: QwtSliderBase(parent,name)
{
    
    //
    // initialize private variables
    //
    setTotalAngle(360);
    setViewAngle(175);
    setInternalBorder(2);
    setUpdateTime(50);
    tickCnt = 10;
    d_borderWidth = 2;
    d_colorCnt = 100;
    d_colors = new QColor[d_colorCnt];
    createColors();
    d_pixmap.resize(10,10);
}

//------------------------------------------------------------
//
//.S	QwtWheel::~QwtWheel
//	Destructor
//
//.s	Syntax
//.f	 QwtWheel::~QwtWheel()
//
//------------------------------------------------------------
QwtWheel::~QwtWheel()
{
    delete[] d_colors;
    
}

//------------------------------------------------------------
//.-
//.S	QwtWheel::createColors
//	Set up the color array for the background pixmap.
//
//.s	Syntax
//.f	void QwtWheel::buildColors()
//
//------------------------------------------------------------
void QwtWheel::createColors()
{
    int i;
    int dh, ds, dv, lh, ls, lv;
    double factor;
    
    QColorGroup g = colorGroup();
    d_colors[0] = g.light();
    d_colors[d_colorCnt - 1] = g.dark();

    d_colors[0].rgb(&lh, &ls, &lv);
    d_colors[d_colorCnt - 1].rgb(&dh, &ds, &dv);
    
    for (i=1; i < d_colorCnt - 1; i++)
    {
	factor = double(i) /  double (d_colorCnt);
	
	d_colors[i].setRgb( lh + int (double(dh - lh) * factor ),
			   ls + int ( double(ds - ls) * factor ),
			   lv + int ( double(dv - lv) * factor ));
    }
    
}

//------------------------------------------------------------
//
//.S	QwtWheel::setTickCnt
//		Adjust the number of grooves in the wheel's surface.
//
//.s	Parameters
//.p	int cnt -- Number of grooves per 360 degrees
//
//.s	Description
//		The number of grooves is limited to 6 <= cnt <= 50.
//		Values outside this range will be clipped.
//		The default value is 10.
//.s	Syntax
//.f	void QwtWheel::setTickCnt(int cnt)
//
//------------------------------------------------------------
void QwtWheel::setTickCnt(int cnt)
{
    if (cnt < 6)
       tickCnt = 6;
    else if (cnt > 50)
       tickCnt = 50;
    else
       tickCnt = cnt;
}


//------------------------------------------------------------
//
//.S	QwtWheel::setInternalBorder
//      Set the internal border width of the wheel.
//
//.s	Parameters
//.p	int w -- border width
//
//.s	Return Type
//		void
//
//.s	Description
//      The internal border must not be smaller than 1
//      and is limited in dependance on the wheel's size.
//      Values outside the allowed range will be clipped.
//      The internal border defaults to 1.
//
//.s	Syntax
//.f	void QwtWheel::setInternalBorder(int w)
//
//------------------------------------------------------------
void QwtWheel::setInternalBorder(int w) 
{
    if(w < 1) 
       d_intBorder = 1;
    else if ((w > width() / 3) || (w > height() / 3))
       d_intBorder = (width() < height()) ? width() / 3 : height() / 3 ;
    else 
       d_intBorder = w;
}

//------------------------------------------------------------
//.-
//.S	QwtWheel::rebuildPixmap
//		Refill the Wheel's background pixmap
//
//.s	Syntax
//.f	void QwtWheel::rebuildPixmap()
//
//------------------------------------------------------------
void QwtWheel::rebuildPixmap()
{

    QPen darkPen, lightPen;
    int rx, ry, rh, rw;
    int hiPos, nFields;
    int i;
    int x1, x2, y1, y2;
    static int allocContext = 0;

#ifndef QT_1_0

    if (allocContext)
       QColor::destroyAllocContext(allocContext);

    allocContext = QColor::enterAllocContext();
#endif	

    nFields = d_colorCnt * 13 / 10;
    hiPos = nFields - d_colorCnt + 1;
    
    QColorGroup g = colorGroup();

    const QRect &r = d_pixmap.rect();

    //
    // initialize pens
    //
    lightPen.setColor(g.light());
    lightPen.setWidth(d_intBorder);
    darkPen.setColor(g.dark());
    darkPen.setWidth(d_intBorder);

    //
    // initialize auxiliary variables
    //

    QPainter p;

    if (p.begin(&d_pixmap))
    {
	if (d_orient == Horizontal)
	{

	    rx = r.x();
	    ry = r.y() + d_intBorder;
	    rh = r.height() - 2* d_intBorder;
	    rw = r.width();
	    //
	    //  draw shaded background
	    //
	    x1 = rx;
	    x2 = rx;
	    for (i=1; i < nFields; i++)
	    {
		x2 = rx + (rw * i) / nFields;
		p.fillRect(x1, ry, x2-x1 + 1 ,rh, d_colors[qwtAbs(i-hiPos)]);
		x1 = x2 + 1;
	    }
	    p.fillRect(x1, ry, rw - (x1 - rx), rh, d_colors[d_colorCnt - 1]);


	    //
	    // draw internal border
	    //
	    p.setPen(lightPen);
	    ry = r.y() + d_intBorder / 2;
	    p.drawLine(r.x(), ry, r.x() + r.width() , ry); 
	    
	    p.setPen(darkPen);
	    ry = r.y() + r.height() - (d_intBorder - d_intBorder / 2);
	    p.drawLine(r.x(), ry , r.x() + r.width(), ry); 

	    
	}
	else
	{

	    rx = r.x() + d_intBorder;
	    ry = r.y();
	    rh = r.height();
	    rw = r.width() - 2 * d_intBorder;

	    //
	    // draw shaded background
	    //
	    y1 = ry;
	    y2 = ry;
	    for (i=1; i < nFields; i++)
	    {
		y2 = ry + (rh * i) / nFields;
		p.fillRect(rx, y1, rw, y2-y1 + 1, d_colors[qwtAbs(i-hiPos)]);
		y1 = y2 + 1;
	    }
	    p.fillRect(rx, y1, rw, rh - (y1 - ry), d_colors[d_colorCnt - 1]);

	    //
	    //	draw internal borders
	    //
	    p.setPen(lightPen);
	    rx = r.x() + d_intBorder / 2;
	    p.drawLine(rx, r.y(), rx, r.y() + r.height()); 
	    
	    p.setPen(darkPen);
	    rx = r.x() + r.width() - (d_intBorder - d_intBorder / 2);
	    p.drawLine(rx, r.y(), rx , r.y() + r.height()); 

	    
	}
    }

    p.end();

#ifndef QT_1_0
    QColor::leaveAllocContext();
#endif
    
}


//------------------------------------------------------------
//
//.S	setTotalAngle
//		Set the total angle which the wheel can be turned.
//
//.s	Parameters
//.p	double angle -- total angle in degrees
//
//
//.s	Description
//		One full turn of the wheel corresponds to an angle of
//		360 degrees. A total angle of n*360 degrees means
//		that the wheel has to be turned n times around its axis
//		to get from the minimum value to the maximum value.
//		The default setting of the total angle is 360 degrees.
//
//.s	Syntax
//.f	void setTotalAngle(double angle)
//
//------------------------------------------------------------
void QwtWheel::setTotalAngle(double angle) 
{
    
    // range check for total angle
    if (angle > 3600) totalAngle = 3600;
    else if (angle < 10)  totalAngle = 10;
    else totalAngle = angle;
    
}

//------------------------------------------------------------
//
//.S	QwtWheel::setOrientation
//		Set the wheel's orientation.
//
//.s	Parameters
//.p	Orientation o --  Orientation. Allowed values are
//			QwtWheel::Horizontal and QwtWheel::Vertical.
//
//.s	Description
//		The default orientation is QwtWheel::Horizontal.
//
//.s	Syntax
//.f	void QwtWheel::setOrientation(Orient o)
//
//------------------------------------------------------------
void QwtWheel::setOrientation(Orientation o)
{
    if (d_orient != o)
    {
	d_orient = o;
	resize(size());
	repaint(FALSE);
    }
    
}


//------------------------------------------------------------
//
//.S	QwtWheel::setViewAngle
//		Specify the visible portion of the wheel.
//
//.s	Parameters
//.p	double angle --			Visible angle in degrees
//
//
//.s	Description
//	You may use this function for fine-tuning the appearance of
//	the wheel. The default value is 175 degrees. The value is
//	limited from 10 to 175 degrees.
//
//.s	Syntax
//.f	void QwtWheel::setViewAngle(double angle)
//
//------------------------------------------------------------
void QwtWheel::setViewAngle(double angle) {
	
	if (angle < 10) viewAngle = 10;
	else if (angle > 175) viewAngle = 175;
	else viewAngle = angle;
	
    }

//-------------------------------------------------------------
//.-
//.S	QwtWheel::drawWheel
//	Redraw the wheel
//
//.s Parameters
//	QPainter *p	-- painter
//	const QRect&r -- contents rectangle
//
//
//-------------------------------------------------------------
void QwtWheel::drawWheel( QPainter *p, const	QRect &r)
{

    QColorGroup g = colorGroup();
    double loValue, hiValue, halfIntv, tickWidth, tickValue, halfSize;
    int tickPos;
    double cnvFactor, sinArc;
    double sign;
    int l2 = 0,l1 = 0, maxPos;


    sign = (minValue() < maxValue()) ? 1.0 : -1.0;

    //
    //	calculate some useful values
    //
    cnvFactor = qwtAbs(totalAngle / (maxValue() - minValue()));
    halfIntv = 0.5 * viewAngle / cnvFactor;
    loValue = value() - halfIntv;
    hiValue = value() + halfIntv;
    tickWidth = 360.0 / double(tickCnt) / cnvFactor;
    sinArc = sin(viewAngle * M_PI / 360.0);
    cnvFactor *= M_PI / 180.0;


    //
    // draw background pixmap
    //
    bitBlt(this, r.x(), r.y(), &d_pixmap, 0, 0);
    
    //
    // draw grooves
    //
    if (d_orient == Horizontal) 
    {
	halfSize = double(r.width()) * 0.5;

	l1 = r.y() + d_intBorder;
	l2 = r.y() + r.height() - d_intBorder - 1;

	// draw one point over the border if border > 1
	if (d_intBorder > 1)
	{
	    l1 --;
	    l2 ++;
	}
	
	maxPos = r.x() + r.width() - 2;
	minPos = r.x() + 2;

	//
	// draw tick marks
	//
	for (tickValue = ceil(loValue / tickWidth) * tickWidth;
	     tickValue < hiValue; tickValue += tickWidth)
	{	
	    //
	    //	calculate position
	    //
	    tickPos = r.x() + r.width()
	       - int( halfSize
		     * (sinArc + sign *  sin((tickValue - value()) * cnvFactor))
		     / sinArc);
	    //
	    // draw vertical line
	    //
	    if ((tickPos <= maxPos) && (tickPos > minPos)) 
	    {
		p->setPen(g.dark());
		p->drawLine(tickPos -1 , l1, tickPos - 1,  l2 );	
		p->setPen(g.light());
		p->drawLine(tickPos, l1, tickPos, l2);	
	    }
	    
	}
    }
    else if (d_orient == Vertical) 
    {
	halfSize = double(r.height()) * 0.5;

	l1 = r.x() + d_intBorder;
	l2 = r.x() + r.width() - d_intBorder - 1;

	if (d_intBorder > 1) 
	{
	    l1--;
	    l2++;
	}

	maxPos = r.y() + r.height() - 2;
	minPos = r.y() + 2;
	
	//
	// draw tick marks
	//
	for (tickValue = ceil(loValue / tickWidth) * tickWidth;
	     tickValue < hiValue; tickValue += tickWidth)
	{		

	    //
	    // calculate position
	    //
	    tickPos = r.y() + r.height()
	       -  int( halfSize
		      * (sinArc + sign *  sin((tickValue - value()) * cnvFactor))
		      / sinArc);
	    
	    //
	    //	draw horizontal line
	    //
	    if ((tickPos <= maxPos) && (tickPos > minPos))
	    {
		p->setPen(g.dark());
		p->drawLine(l1, tickPos - 1 ,l2, tickPos - 1);	
		p->setPen(g.light());
		p->drawLine(l1, tickPos, l2, tickPos);	
	    }
	    
	}
    }
}


//------------------------------------------------------------
//.-
//.S	QwtWheel::getValue
//	Determine the value corresponding to a specified point
//
//.s	Parameters
//.p	const QPoint &p -- point
//
//.s	Return Type
//		double
//
//.s	Description
//	This function is only called by the QwtSliderBase class.
//
//.s	Syntax
//.f	double QwtWheel::getValue(const QPoint &p)
//
//------------------------------------------------------------
double QwtWheel::getValue( const QPoint &p) 
{  
    double arc;
    int dx,w;

    QRect &r = d_sliderRect;
    
    //
    // return the value corresponding to point p.
    //
    
    if (d_orient == Vertical)
    {
	w = r.height();
	if (p.y() > r.y() + w)
	   dx = w;
	else if (p.y() < r.y())
	   dx = 0;
	else
	   dx = p.y() - r.y();
    }
    else
    {
	w = r.width();
	if (p.x() > r.x() + w)
	   dx =  w;
	else if (p.x() < r.x())
	   dx = 0;
	else
	   dx = p.x() - r.x();
    }
    
    arc = asin( double( 2 * dx - w ) / double(w)
	       * sin(viewAngle * M_PI / 360.0 ))
       /  M_PI * 180.0;
    
    //
    // If this seems wrong to you, you're right...
    // But the QwtSliderBase class only needs the
    // correct offset here, not the real value.
    //
    return arc / totalAngle       
       * (maxValue() - minValue());   
    
    
}

//------------------------------------------------------------
//.-
//.S	QwtWheel::resizeEvent
//		Qt Resize Event
//
//.s	Parameters
//.p	QResizeEvent *e 
//
//.s	Syntax
//.f	void QwtWheel::resizeEvent(QResizeEvent *e )
//
//------------------------------------------------------------
void QwtWheel::resizeEvent(QResizeEvent *e )
{
    const QSize &s = e->size();
    d_sliderRect.setRect(rect().x() + d_borderWidth, rect().y() + d_borderWidth,
			 s.width() - 2*d_borderWidth, s.height() - 2*d_borderWidth);
    
    d_pixmap.resize(d_sliderRect.width(), d_sliderRect.height());
    
    rebuildPixmap();
}

//------------------------------------------------------------
//.-
//.S	QwtWheel::paintEvent
//	Qt paint event
//
//.s	Parameters
//.p	QPaintEvent *e
//
//.s	Syntax
//.f	void QwtWheel::paintEvent(QPaintEvent *e)
//
//------------------------------------------------------------
void QwtWheel::paintEvent(QPaintEvent *e)
{
    QPainter p;
    if(p.begin(this))
    {
	qDrawShadePanel(&p,
			rect().x(), rect().y(),
			rect().width(), rect().height(),
			colorGroup(), TRUE, d_borderWidth);
	
	drawWheel(&p, d_sliderRect);
    }
    p.end();
    
}

//------------------------------------------------------------
//.-
//.S	QwtWheel::valueChange
//	Notify value change	
//
//.s	Description
//	Called by QwtSliderBase
//
//.s	Syntax
//.f	void QwtWheel::valueChange()
//
//------------------------------------------------------------
void QwtWheel::valueChange()
{
    repaint(FALSE);
    QwtSliderBase::valueChange();
}


//------------------------------------------------------------
//.-
//.S	QwtWheel::getScrollMode
//	Determine the scrolling mode and direction corresponding
//	to a specified point
//
//.s	Parameters
//.p	 const QPoint &p -- point
//       int &scrollMode	-- scrolling mode
//	 int &direction		-- direction
//
//.s Description
//	Called by QwtSliderbase
//
//.s	Syntax
//.f	 void QwtWheel::getScrollMode( const QPoint &p, int &scrollMode, int &direction)
//
//------------------------------------------------------------
void QwtWheel::getScrollMode( const QPoint &p, int &scrollMode, int &direction) 
{
    //
    // set scrollMode and direction
    //
    if (d_sliderRect.contains(p))
       scrollMode = ScrMouse;
    else
       scrollMode = ScrNone;
    
    direction = 0;
    
}



//------------------------------------------------------------
//
//.S	QwtWheel::setMass
//	Set the mass of the wheel
//
//.s	Parameters
//.p	double val	-	the wheel's mass
//
//.s	Description
//	Assigning a mass turns the wheel into a flywheel.
//
//.s	Syntax
//.f	void QwtWheel::setMass(double val)
//
//------------------------------------------------------------
void QwtWheel::setMass(double val)
{
    QwtSliderBase::setMass(val);
}































